#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <unistd.h>
#include <fcntl.h>
#include <linux/uinput.h>

#include "gamepad.h"
#include "panic.h"


void waitmouse(int mouse, struct input_event* ie) {
	while (1) {
		read(mouse, ie, sizeof(struct input_event));
		if (ie->type != EV_SYN)
			return;
	}
}

int clamp(int num) {
	// Bruh
	if (num > 33000)
		return 33000;
	if (num < -33000)
		return -33000;
	return num;
}

int cut(int num, int amount) {
	if (num > -amount && num < amount)
		return 0;
	return num;
}

int drop(int num, int amount) {
	if (num > 0)
		return num - amount;
	if (num < 0)
		return num + amount;
	return 0;
}

int main(int argc, char* argv[]) {
	if (argc == 0) {
		fprintf(stderr, "????\n");
		return 255;
	}
	if (argc != 5) {
		printf("Usage: %s devicepath scale_axis scale_wheel deadzone\n", argv[0]);
		return 1;
	}
	struct input_event ie;
	int axis_x = 0;
	int axis_y = 0;
	int axis_w = 0;
	int scale_axis = strtol(argv[2], NULL, 10);
	int scale_wheel = strtol(argv[3], NULL, 10);
	int deadzone = strtol(argv[3], NULL, 10);
	int drop_axis = 0;
	int drop_wheel = 0;

	char* device = argv[1];
	int mouse = open(device, 0);
	if (!mouse)
		panic("Unable to open device.");

	int gamepad = setupgamepad();

	while (1) {
		waitmouse(mouse, &ie);
		if (ie.type == EV_REL && ie.code == REL_X)
			axis_x = drop(axis_x + ie.value * scale_axis, drop_axis);
		if (ie.type == EV_REL && ie.code == REL_Y)
			axis_y = drop(axis_y + ie.value * scale_axis, drop_axis);
		if (ie.type == EV_REL && ie.code == REL_WHEEL)
			axis_w = drop(axis_w + ie.value * scale_wheel, drop_wheel);
		axis_x = clamp(axis_x);
		axis_y = clamp(axis_y);
		axis_w = clamp(axis_w);
		sendaxis(gamepad, ABS_X, cut(axis_x, deadzone));
		sendaxis(gamepad, ABS_Y, cut(axis_y, deadzone));
		sendaxis(gamepad, ABS_HAT1X, axis_w);
		if (ie.type == EV_KEY && ie.code == BTN_LEFT)
			sendkey(gamepad, BTN_TL2, ie.value);
		if (ie.type == EV_KEY && ie.code == BTN_MIDDLE)
			sendkey(gamepad, BTN_SOUTH, ie.value);
		if (ie.type == EV_KEY && ie.code == BTN_RIGHT)
			sendkey(gamepad, BTN_TR2, ie.value);
	}
	return 0;
}
